{#-
  This is an alias file: nb_NO -> nb
  See https://squidfunk.github.io/mkdocs-material/setup/changing-the-language/#custom-translations
-#}
{% import "partials/languages/nb.html" as nb %}
{% macro t(key) %}{{ nb.t(key) }}{% endmacro %}
