/*
 * SPDX-FileCopyrightText: 2024 Nicco Kunzmann and Open Web Calendar Contributors <https://open-web-calendar.quelltext.eu/>
 *
 * SPDX-License-Identifier: GPL-2.0-only
 */

header, header a {
    text-align: center;
    color: white;
    background-color: black;
    padding: 0.66em;
    text-decoration: none;
}

body {
    margin: 0px;
    padding:0px;
}

footer {
  border-color: black;
  border-width: 0px;
  border-top-width: 1px;
  border-style: solid;
  margin-top: 0.3em;
  padding-top: 0.3em;
  color: gray;
  padding-left: 1em;
  padding-right: 1em;
  padding-bottom: 1em;
}

footer a {
  color: gray;
}

.languages {
  text-align: center;
}

.languages a {
  color: gray;
  text-decoration: none;
  margin: 0.2em;
  display: inline-block;
}

.languages a:hover {
  text-decoration: underline;
}

.languages a.current {
  color: black;
}

main {
    padding: 1em;
    max-width: 60em;
    margin-right: auto;
    margin-left: auto;
}

.code {
    padding: 1em;
    background-color: #eee;
    border-radius: 0.66em;
    overflow: hidden;
}

#open-web-calendar {
}

.calendar-url-input {
    width: 80%;
}

#open-web-calendar {
    width: 100%;
    height: 600px;
}

#css-input {
  width: 100%;
}

.STATUS-TENTATIVE {
  font-style: italic;
}

.STATUS-CONFIRMED {
  font-weight: bold;
}

.STATUS-CANCELLED {
  text-decoration: line-through;
}

.css-class-example {
  display: inline-block;
}
