/*
 * SPDX-FileCopyrightText: 2024 Nicco Kunzmann and Open Web Calendar Contributors <https://open-web-calendar.quelltext.eu/>
 *
 * SPDX-License-Identifier: GPL-2.0-only
 */

/* Styling of the dhtmlx.html file.
 *
 */

.status-window {
    position: absolute;
    right: 0px;
    bottom: 0px;
    z-index: 60;
}

.status-window .item {
    min-width: 1em;
    min-height: 1em;
    padding: 0.2em;
    text-align: center;
    text-decoration: none;
    display: inline-block;
}

#infoIcon {
    background-color: rgba(0,0,0,0.1);
}

#errorStatusIcon {
    display: none;
    background-color: #ff7777;
}

#errorStatusIcon.onError {
    display: inline-block;
    z-index: 150;
}

#errorWindow {
    background-color: rgba(255, 208, 208, 0.9);
    z-index: 50;
}

.fullsize {
    top: 0px;
    left: 0px;
    width: 100%;
    min-height: 100%;
}

#errorWindow .closeButton {
    position: absolute;
    top: 0px;
    right: 0px;
    min-width: 1em;
    min-height: 1em;
    padding: 0.2em;
    text-align: center;
    background-color: #ff7777;
    color: black;
    border-bottom-left-radius: 0.2em;
    z-index: 100;
}

#errorWindow iframe {
    width: 100%;
    border-width: 0px;
}

#errorWindow .item {
    border-color: #ff7777;
    border-style: solid;
    border-bottom-width: 0.2em;
    text-decoration: none;
}

.hidden, #scheduler_here .error {
    display: none;
}

#errorWindow .errorFrame {
    /* The iframe in case of an error. */
    height: 40em;
}

#loader {
    height: 100px;
    z-index: 40;
    position: absolute;
}

.dhx_cal_qi_controls {
  /* hide the delete and details button */
  display: none
}

div.dhx_agenda_line > span {
    /* fixes vanishing of calendar entries in agenda view at narrow mobile viewport sizes */
	display: block ruby !important;
	text-overflow: ellipsis;
	overflow: hidden;
}

@media only screen and (max-width: 480px) {
    .event {
        height: 4.17vw;
        font-size: 2.92vw;
    }

    .dhx_month_head {
        height: 6.5vw;
        font-size: 2.5vw;
        line-height: 3.75vw;
    }
}

/* The possible link in the title. */
.dhx_cal_qi_tcontent a {
  color: inherit;
}

/* Scheduler added a resize bar at the bottom. We do not allow edits so we remove it. */

.dhx_event_resize.dhx_footer {
    display: none;
}
