<!--
SPDX-FileCopyrightText: 2024 Nicco Kunzmann and Open Web Calendar Contributors <https://open-web-calendar.quelltext.eu/>

SPDX-License-Identifier: GPL-2.0-only
-->

<!DOCTYPE html>
<html>
<head>
  <title>Forbidden - 403</title>
</head>
<body>

<h1>Forbidden</h1>
<p>You requested the Open Web Calendar for the hostname {{ hostname }} but only these host names are allowed: {{ allowed_hosts }}</p>

</body>
</html>
